/**
 * Qt6 Common library
 *
 * Copyright (C) 2026 The Qt Company Ltd.
 * SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 *
 * NOTE: This class is auto generated by OpenAPI Generator (https://openapi-generator.tech).
 * https://openapi-generator.tech
 * Do not edit the class manually.
 */

#include "qoaienum.h"

QT_BEGIN_NAMESPACE

namespace QtOpenApiCommon {

QOAIEnum::QOAIEnum() = default;

QOAIEnum::~QOAIEnum() = default;

} // namespace QtOpenApiCommon

QT_END_NAMESPACE
