/****************************************************************************
** Meta object code from reading C++ file 'qoaibaseapi.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../common/qoaibaseapi.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qoaibaseapi.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QtOpenApiCommon11QOAIBaseApiE_t {};
} // unnamed namespace

template <> constexpr inline auto QtOpenApiCommon::QOAIBaseApi::qt_create_metaobjectdata<qt_meta_tag_ZN15QtOpenApiCommon11QOAIBaseApiE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QtOpenApiCommon::QOAIBaseApi",
        "operations",
        "",
        "serverConfigurations",
        "QList<QOAIServerConfiguration>",
        "operation",
        "setServerVariable",
        "ServerError",
        "qsizetype",
        "serverIndex",
        "variable",
        "value",
        "setServer",
        "activeServer",
        "std::optional<qsizetype>",
        "setApiKey",
        "apiKeyName",
        "apiKey",
        "std::optional<QString>",
        "name",
        "setBearerToken",
        "token",
        "bearerToken",
        "setUsername",
        "username",
        "setPassword",
        "password",
        "setTimeOut",
        "std::chrono::milliseconds",
        "timeOut",
        "setSslConfiguration",
        "QSslConfiguration",
        "config",
        "sslConfiguration",
        "setWorkingDirectory",
        "path",
        "workingDirectory",
        "setRestAccessManager",
        "QRestAccessManager*",
        "ram",
        "restAccessManager",
        "setNetworkRequestFactory",
        "QNetworkRequestFactory",
        "factory",
        "networkRequestFactory",
        "setHeader",
        "QHttpHeaders::WellKnownHeader",
        "key",
        "QAnyStringView",
        "setRequestCompressionEnabled",
        "enabled",
        "requestCompressionEnabled",
        "setResponseCompressionEnabled",
        "responseCompressionEnabled",
        "setFileConflictPolicy",
        "FileConflictPolicy",
        "policy",
        "fileConflictPolicy",
        "errorString",
        "error"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'operations'
        QtMocHelpers::SlotData<QList<QString>() const>(1, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'serverConfigurations'
        QtMocHelpers::SlotData<QList<QOAIServerConfiguration>(const QString &) const>(3, 2, QMC::AccessPublic, 0x80000000 | 4, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'setServerVariable'
        QtMocHelpers::SlotData<ServerError(const QString &, qsizetype, const QString &, const QString &)>(6, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 8, 9 }, { QMetaType::QString, 10 }, { QMetaType::QString, 11 },
        }}),
        // Slot 'setServer'
        QtMocHelpers::SlotData<ServerError(const QString &, qsizetype)>(12, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 5 }, { 0x80000000 | 8, 9 },
        }}),
        // Slot 'activeServer'
        QtMocHelpers::SlotData<std::optional<qsizetype>(const QString &) const>(13, 2, QMC::AccessPublic, 0x80000000 | 14, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'setApiKey'
        QtMocHelpers::SlotData<void(const QString &, const QString &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 16 }, { QMetaType::QString, 17 },
        }}),
        // Slot 'apiKey'
        QtMocHelpers::SlotData<std::optional<QString>(const QString &) const>(17, 2, QMC::AccessPublic, 0x80000000 | 18, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'setBearerToken'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Slot 'bearerToken'
        QtMocHelpers::SlotData<QString() const>(22, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setUsername'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 24 },
        }}),
        // Slot 'username'
        QtMocHelpers::SlotData<QString() const>(24, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setPassword'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 26 },
        }}),
        // Slot 'password'
        QtMocHelpers::SlotData<QString() const>(26, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setTimeOut'
        QtMocHelpers::SlotData<void(std::chrono::milliseconds)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 28, 29 },
        }}),
        // Slot 'setSslConfiguration'
        QtMocHelpers::SlotData<void(QSslConfiguration)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'sslConfiguration'
        QtMocHelpers::SlotData<QSslConfiguration() const>(33, 2, QMC::AccessPublic, 0x80000000 | 31),
        // Slot 'setWorkingDirectory'
        QtMocHelpers::SlotData<void(const QString &)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 35 },
        }}),
        // Slot 'workingDirectory'
        QtMocHelpers::SlotData<QString() const>(36, 2, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setRestAccessManager'
        QtMocHelpers::SlotData<void(QRestAccessManager *)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 38, 39 },
        }}),
        // Slot 'restAccessManager'
        QtMocHelpers::SlotData<QRestAccessManager *() const>(40, 2, QMC::AccessPublic, 0x80000000 | 38),
        // Slot 'setNetworkRequestFactory'
        QtMocHelpers::SlotData<void(QNetworkRequestFactory)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 42, 43 },
        }}),
        // Slot 'networkRequestFactory'
        QtMocHelpers::SlotData<QNetworkRequestFactory() const>(44, 2, QMC::AccessPublic, 0x80000000 | 42),
        // Slot 'setHeader'
        QtMocHelpers::SlotData<void(QHttpHeaders::WellKnownHeader, QAnyStringView)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 46, 47 }, { 0x80000000 | 48, 11 },
        }}),
        // Slot 'setHeader'
        QtMocHelpers::SlotData<void(QAnyStringView, QAnyStringView)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 48, 47 }, { 0x80000000 | 48, 11 },
        }}),
        // Slot 'setRequestCompressionEnabled'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 50 },
        }}),
        // Slot 'requestCompressionEnabled'
        QtMocHelpers::SlotData<bool() const>(51, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setResponseCompressionEnabled'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 50 },
        }}),
        // Slot 'responseCompressionEnabled'
        QtMocHelpers::SlotData<bool() const>(53, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFileConflictPolicy'
        QtMocHelpers::SlotData<void(FileConflictPolicy)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'fileConflictPolicy'
        QtMocHelpers::SlotData<FileConflictPolicy() const>(57, 2, QMC::AccessPublic, 0x80000000 | 55),
        // Slot 'errorString'
        QtMocHelpers::SlotData<QString(ServerError) const>(58, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 7, 59 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QOAIBaseApi, qt_meta_tag_ZN15QtOpenApiCommon11QOAIBaseApiE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QtOpenApiCommon::QOAIBaseApi::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QtOpenApiCommon11QOAIBaseApiE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QtOpenApiCommon11QOAIBaseApiE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QtOpenApiCommon11QOAIBaseApiE_t>.metaTypes,
    nullptr
} };

void QtOpenApiCommon::QOAIBaseApi::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QOAIBaseApi *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QList<QString> _r = _t->operations();
            if (_a[0]) *reinterpret_cast<QList<QString>*>(_a[0]) = std::move(_r); }  break;
        case 1: { QList<QOAIServerConfiguration> _r = _t->serverConfigurations((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<QOAIServerConfiguration>*>(_a[0]) = std::move(_r); }  break;
        case 2: { ServerError _r = _t->setServerVariable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[4])));
            if (_a[0]) *reinterpret_cast<ServerError*>(_a[0]) = std::move(_r); }  break;
        case 3: { ServerError _r = _t->setServer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qsizetype>>(_a[2])));
            if (_a[0]) *reinterpret_cast<ServerError*>(_a[0]) = std::move(_r); }  break;
        case 4: { std::optional<qsizetype> _r = _t->activeServer((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::optional<qsizetype>*>(_a[0]) = std::move(_r); }  break;
        case 5: _t->setApiKey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 6: { std::optional<QString> _r = _t->apiKey((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::optional<QString>*>(_a[0]) = std::move(_r); }  break;
        case 7: _t->setBearerToken((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: { QString _r = _t->bearerToken();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->setUsername((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: { QString _r = _t->username();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 11: _t->setPassword((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: { QString _r = _t->password();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->setTimeOut((*reinterpret_cast<std::add_pointer_t<std::chrono::milliseconds>>(_a[1]))); break;
        case 14: _t->setSslConfiguration((*reinterpret_cast<std::add_pointer_t<QSslConfiguration>>(_a[1]))); break;
        case 15: { QSslConfiguration _r = _t->sslConfiguration();
            if (_a[0]) *reinterpret_cast<QSslConfiguration*>(_a[0]) = std::move(_r); }  break;
        case 16: _t->setWorkingDirectory((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 17: { QString _r = _t->workingDirectory();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->setRestAccessManager((*reinterpret_cast<std::add_pointer_t<QRestAccessManager*>>(_a[1]))); break;
        case 19: { QRestAccessManager* _r = _t->restAccessManager();
            if (_a[0]) *reinterpret_cast<QRestAccessManager**>(_a[0]) = std::move(_r); }  break;
        case 20: _t->setNetworkRequestFactory((*reinterpret_cast<std::add_pointer_t<QNetworkRequestFactory>>(_a[1]))); break;
        case 21: { QNetworkRequestFactory _r = _t->networkRequestFactory();
            if (_a[0]) *reinterpret_cast<QNetworkRequestFactory*>(_a[0]) = std::move(_r); }  break;
        case 22: _t->setHeader((*reinterpret_cast<std::add_pointer_t<QHttpHeaders::WellKnownHeader>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAnyStringView>>(_a[2]))); break;
        case 23: _t->setHeader((*reinterpret_cast<std::add_pointer_t<QAnyStringView>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QAnyStringView>>(_a[2]))); break;
        case 24: _t->setRequestCompressionEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: { bool _r = _t->requestCompressionEnabled();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setResponseCompressionEnabled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: { bool _r = _t->responseCompressionEnabled();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 28: _t->setFileConflictPolicy((*reinterpret_cast<std::add_pointer_t<FileConflictPolicy>>(_a[1]))); break;
        case 29: { FileConflictPolicy _r = _t->fileConflictPolicy();
            if (_a[0]) *reinterpret_cast<FileConflictPolicy*>(_a[0]) = std::move(_r); }  break;
        case 30: { QString _r = _t->errorString((*reinterpret_cast<std::add_pointer_t<ServerError>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 14:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSslConfiguration >(); break;
            }
            break;
        }
    }
}

const QMetaObject *QtOpenApiCommon::QOAIBaseApi::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QtOpenApiCommon::QOAIBaseApi::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QtOpenApiCommon11QOAIBaseApiE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QtOpenApiCommon::QOAIBaseApi::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 31)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 31;
    }
    return _id;
}
QT_WARNING_POP
